@@ -8,7 +8,7 @@ r'''more-reasonable core functionality for numpy
 
     >>> a   = np.arange(6).reshape(2,3)
     >>> b   = a + 100
-    >>> row = a[0,:]
+    >>> row = a[0,:] + 1000
 
     >>> a
     array([[0, 1, 2],
@@ -59,10 +59,10 @@ have used PDL in the past.
 Instead of writing a new module (this module), it would be really nice to simply
 patch numpy to give everybody the more reasonable behavior. I'd be very happy to
 do that, but the issues lie with some very core functionality, and any changes
-in behavior would likely break existing code. Any comments in how to achieve
-better behaviors in a less forky manner as welcome.
+in behavior would break existing code. Any comments in how to achieve better
+behaviors in a less forky manner are welcome.
 
-Finally, if the system DOES make sense in some way that I'm simply not
+Finally, if the existing system DOES make sense in some way that I'm simply not
 understanding, I'm happy to listen. I have no intention to disparage anyone or
 anything; I just want a more usable system for numerical computations.
 
@@ -87,8 +87,8 @@ identically-sized vectors (1-dimensional arrays) and returns a scalar
 arrays of shape (2,3,4), compute the 6 inner products of length-4 each, and
 report the output in an array of shape (2,3). Numpy puts the most-significant
 dimension at the end, which is why this isn't 12 inner products of length-2
-each. This is a semi-arbitrary design choice, which could have been made
-differently: PDL puts the most-significant dimension at the front, for instance.
+each. This is an arbitrary design choice, which could have been made
+differently: PDL puts the most-significant dimension at the front.
 
 The user doesn't choose whether to use broadcasting or not: some functions
 support it, and some do not. In PDL, broadcasting (called "threading" in that
@@ -101,11 +101,12 @@ By contrast, in numpy very few functions know how to broadcast. On top of that,
 the documentation is usually silent about the broadcasting status of a function
 in question. And on top of THAT, broadcasting rules state that an array of
 dimensions (n,m) is functionally identical to one of dimensions
-(1,1,1,....1,n,m). However, many numpy functions have special-case rules to
-create different behaviors for inputs with different numbers of dimensions, and
-this creates unexpected results. The effect of all this is a messy situation
-where the user is often not sure of the exact behavior of the functions they're
-calling, and trial and error is required to make the system do what one wants.
+(1,1,1,....1,n,m). Sadly, numpy does not respect its own broadcasting rules, and
+many functions have special-case logic to create different behaviors for inputs
+with different numbers of dimensions; and this creates unexpected results. The
+effect of all this is a messy situation where the user is often not sure of the
+exact behavior of the functions they're calling, and trial and error is required
+to make the system do what one wants.
 
 *** Solution
 This module contains functionality to make any arbitrary function broadcastable.
@@ -518,6 +519,12 @@ Broadcast-aware inner product. Identical to dot
 **** outer
 Broadcast-aware outer product.
 
+**** norm2
+Broadcast-aware 2-norm. norm2(x) is identical to inner(x,x)
+
+**** trace
+Broadcast-aware trace.
+
 **** matmult
 Broadcast-aware matrix multiplication
 
@@ -533,7 +540,7 @@ import inspect
 from distutils.version import StrictVersion
 
 # setup.py assumes the version is a simple string in '' quotes
-__version__ = '0.15'
+__version__ = '0.17'
 
 # object needed for fancy slices. m[:] is exactly the same as m[_colon], but
 # '_colon' is a normal python object, so it can be manipulated in ways that ':'
@@ -1720,7 +1727,7 @@ def dot(a, b, out=None):
         array([5, 6, 7])
 
         >>> nps.dot(a,b)
-        array(20)
+        20
 
     this is identical to numpysane.inner(). for a conjugating version of this
     function, use nps.vdot(). note that the numpy dot() has some special
@@ -1812,6 +1819,62 @@ def outer(a, b, out=None):
     out.setfield(np.outer(a,b), out.dtype)
     return out
 
+# Note that this explicitly isn't done with @broadcast_define. Instead I
+# implement the internals with core numpy routines. The advantage is that these
+# are some of very few numpy functions that support broadcasting, and they do so
+# on the C level, so their broadcasting loop is FAST. Much more so than my
+# current @broadcast_define loop
+def norm2(a, **kwargs):
+    r'''Broadcast-aware 2-norm. norm2(x) is identical to inner(x,x)
+
+    Synopsis:
+
+        >>> import numpy as np
+        >>> import numpysane as nps
+
+        >>> a = np.arange(3)
+        >>> a
+        array([0, 1, 2])
+
+        >>> nps.norm2(a)
+        5
+
+    This is a convenience function to compute a 2-norm
+
+    '''
+    return inner(a,a, **kwargs)
+
+@broadcast_define( (('n','n',),), prototype_output=() )
+def trace(a):
+    r'''Broadcast-aware trace
+
+    Synopsis:
+
+        >>> import numpy as np
+        >>> import numpysane as nps
+
+        >>> a = np.arange(3*4*4).reshape(3,4,4)
+        >>> a
+        array([[[ 0,  1,  2,  3],
+                [ 4,  5,  6,  7],
+                [ 8,  9, 10, 11],
+                [12, 13, 14, 15]],
+
+               [[16, 17, 18, 19],
+                [20, 21, 22, 23],
+                [24, 25, 26, 27],
+                [28, 29, 30, 31]],
+
+               [[32, 33, 34, 35],
+                [36, 37, 38, 39],
+                [40, 41, 42, 43],
+                [44, 45, 46, 47]]])
+
+        >>> nps.trace(a)
+        array([ 30,  94, 158])
+    '''
+    return np.trace(a)
+
 # Could be implemented with a simple loop around np.dot():
 #
 #     @broadcast_define( (('n', 'm'), ('m', 'l')), prototype_output=('n','l'), out_kwarg='out' )
